@@ -34,9 +34,9 @@
  */
 #define __TBB_TODO 0
 
-/*Check which standard library we use on macOS*.*/
-/*__TBB_SYMBOL is defined only while processing exported symbols list where C++ is not allowed.*/
-#if !defined(__TBB_SYMBOL) && (__APPLE__ || __ANDROID__)
+/* Check which standard library we use. */
+/* __TBB_SYMBOL is defined only while processing exported symbols list where C++ is not allowed. */
+#if !defined(__TBB_SYMBOL) && !__TBB_CONFIG_PREPROC_ONLY
     #include <cstddef>
 #endif
 
@@ -50,13 +50,13 @@
 // e.g. it should be set to 40902 for libstdc++ coming with GCC 4.9.2.
 #ifdef TBB_USE_GLIBCXX_VERSION
 #define __TBB_GLIBCXX_VERSION TBB_USE_GLIBCXX_VERSION
-#else
+#elif __GLIBCPP__ || __GLIBCXX__
 #define __TBB_GLIBCXX_VERSION __TBB_GCC_VERSION
 //TODO: analyze __GLIBCXX__ instead of __TBB_GCC_VERSION ?
 #endif
 
 #if __clang__
-    /**according to clang documentation version can be vendor specific **/
+    /** according to clang documentation, version can be vendor specific **/
     #define __TBB_CLANG_VERSION (__clang_major__ * 10000 + __clang_minor__ * 100 + __clang_patchlevel__)
 #endif
 
@@ -108,6 +108,14 @@
     #define __INTEL_COMPILER 1210
 #endif
 
+#if __clang__ && !__INTEL_COMPILER
+#define __TBB_USE_OPTIONAL_RTTI __has_feature(cxx_rtti)
+#elif defined(_CPPRTTI)
+#define __TBB_USE_OPTIONAL_RTTI 1
+#else
+#define __TBB_USE_OPTIONAL_RTTI (__GXX_RTTI || __RTTI || __INTEL_RTTI__)
+#endif
+
 #if __TBB_GCC_VERSION >= 40400 && !defined(__INTEL_COMPILER)
     /** warning suppression pragmas available in GCC since 4.4 **/
     #define __TBB_GCC_WARNING_SUPPRESSION_PRESENT 1
@@ -124,7 +132,7 @@
    support added.
  */
 
-/** C++11 mode detection macros for Intel(R) C++ compiler (enabled by -std=c++XY option):
+/** C++11 mode detection macros for Intel(R) C++ Compiler (enabled by -std=c++XY option):
     __INTEL_CXX11_MODE__ for version >=13.0 (not available for ICC 15.0 if -std=c++14 is used),
     __STDC_HOSTED__ for version >=12.0 (useful only on Windows),
     __GXX_EXPERIMENTAL_CXX0X__ for version >=12.0 on Linux and macOS. **/
@@ -134,7 +142,7 @@
 #endif
 
 // Intel(R) C++ Compiler offloading API to the Intel(R) Graphics Technology presence macro
-// TODO: add support for ICC 15.00 _GFX_enqueue API and then decrease Intel compiler supported version
+// TODO: add support for ICC 15.00 _GFX_enqueue API and then decrease Intel C++ Compiler supported version
 // TODO: add linux support and restict it with (__linux__ && __TBB_x86_64 && !__ANDROID__) macro
 #if __INTEL_COMPILER >= 1600 && _WIN32
 #define __TBB_GFX_PRESENT 1
@@ -170,7 +178,7 @@
     #define __TBB_STATIC_ASSERT_PRESENT                     (__INTEL_CXX11_MODE__ || _MSC_VER >= 1600)
     #define __TBB_CPP11_TUPLE_PRESENT                       (_MSC_VER >= 1600 || __GXX_EXPERIMENTAL_CXX0X__ && (__TBB_GLIBCXX_VERSION >= 40300 || _LIBCPP_VERSION))
     #if (__clang__ && __INTEL_COMPILER > 1400)
-        /* Older versions of Intel Compiler do not have __has_include */
+        /* Older versions of Intel C++ Compiler do not have __has_include */
         #if (__has_feature(__cxx_generalized_initializers__) && __has_include(<initializer_list>))
             #define __TBB_INITIALIZER_LISTS_PRESENT         1
         #endif
@@ -244,7 +252,7 @@
     #define __TBB_ALIGNAS_PRESENT                           (__GXX_EXPERIMENTAL_CXX0X__ && __TBB_GCC_VERSION >= 40800)
     #define __TBB_CPP11_TEMPLATE_ALIASES_PRESENT            (__GXX_EXPERIMENTAL_CXX0X__ && __TBB_GCC_VERSION >= 40700)
 #elif _MSC_VER
-    // These definitions are also used with Intel Compiler in "default" mode; see a comment above.
+    // These definitions are also used with Intel C++ Compiler in "default" mode; see a comment above.
 
     #define __TBB_CPP11_VARIADIC_TEMPLATES_PRESENT          (_MSC_VER >= 1800)
     // Contains a workaround for ICC 13
@@ -304,6 +312,9 @@
                                             __GXX_EXPERIMENTAL_CXX0X__ && __TBB_GLIBCXX_VERSION >= 40700 && !(__TBB_GLIBCXX_VERSION == 40700 && __TBB_DEFINE_MIC))
 #define __TBB_MAKE_EXCEPTION_PTR_PRESENT   (__TBB_EXCEPTION_PTR_PRESENT && (_MSC_VER >= 1700 || __TBB_GLIBCXX_VERSION >= 40600 || _LIBCPP_VERSION))
 
+// Due to libc++ limitations in C++03 mode, do not pass rvalues to std::make_shared()
+#define __TBB_CPP11_SMART_POINTERS_PRESENT ( _MSC_VER >= 1600 || _LIBCPP_VERSION || ((__cplusplus >= 201103L || __GXX_EXPERIMENTAL_CXX0X__) && (__TBB_GLIBCXX_VERSION>=40500 || __TBB_GLIBCXX_VERSION>=40400 && __TBB_USE_OPTIONAL_RTTI)) )
+
 #define __TBB_CPP11_FUTURE_PRESENT (_MSC_VER >= 1700 || __TBB_GLIBCXX_VERSION >= 40600 && _GXX_EXPERIMENTAL_CXX0X__ || _LIBCPP_VERSION)
 
 // std::swap is in <utility> only since C++11, though MSVC had it at least since VS2005
@@ -361,13 +372,13 @@
 /*
 There are four cases that are supported:
   1. "_DEBUG is undefined" means "no debug";
-  2. "_DEBUG defined to something that is evaluated to 0 (the "garbage" is also evaluated to 0 [cpp.cond])" means "no debug";
-  3. "_DEBUG defined to something that is evaluated to non-zero value" means "debug";
+  2. "_DEBUG defined to something that is evaluated to 0" (including "garbage", as per [cpp.cond]) means "no debug";
+  3. "_DEBUG defined to something that is evaluated to a non-zero value" means "debug";
   4. "_DEBUG defined to nothing (empty)" means "debug".
 */
 #ifdef _DEBUG
 // Check if _DEBUG is empty.
-#define __TBB_IS__DEBUG_EMPTY (__TBB_IS_MACRO_EMPTY(_DEBUG,)==__TBB_MACRO_EMPTY)
+#define __TBB_IS__DEBUG_EMPTY (__TBB_IS_MACRO_EMPTY(_DEBUG,IGNORED)==__TBB_MACRO_EMPTY)
 #if __TBB_IS__DEBUG_EMPTY
 #define TBB_USE_DEBUG 1
 #else
@@ -391,7 +402,7 @@ There are four cases that are supported:
 #define TBB_USE_PERFORMANCE_WARNINGS TBB_PERFORMANCE_WARNINGS
 #else
 #define TBB_USE_PERFORMANCE_WARNINGS TBB_USE_DEBUG
-#endif /* TBB_PEFORMANCE_WARNINGS */
+#endif /* TBB_PERFORMANCE_WARNINGS */
 #endif /* TBB_USE_PERFORMANCE_WARNINGS */
 
 #if __TBB_DEFINE_MIC
@@ -410,18 +421,10 @@ There are four cases that are supported:
     #define TBB_USE_EXCEPTIONS 1
 #endif
 
-#if __clang__ && !__INTEL_COMPILER
-#define __TBB_USE_OPTIONAL_RTTI __has_feature(cxx_rtti)
-#elif defined(_CPPRTTI)
-#define __TBB_USE_OPTIONAL_RTTI 1
-#else
-#define __TBB_USE_OPTIONAL_RTTI (__GXX_RTTI || __RTTI || __INTEL_RTTI__)
-#endif
-
 #ifndef TBB_IMPLEMENT_CPP0X
 /** By default, use C++11 classes if available **/
     #if __clang__
-        /* Old versions of Intel Compiler do not have __has_include or cannot use it in #define */
+        /* Old versions of Intel C++ Compiler do not have __has_include or cannot use it in #define */
         #if (__INTEL_COMPILER && (__INTEL_COMPILER < 1500 || __INTEL_COMPILER == 1500 && __INTEL_COMPILER_UPDATE <= 1))
             #define TBB_IMPLEMENT_CPP0X (__cplusplus < 201103L || !_LIBCPP_VERSION)
         #else
@@ -507,7 +510,7 @@ There are four cases that are supported:
 #endif /* __TBB_SLEEP_PERMISSION */
 
 #ifndef __TBB_TASK_ISOLATION
-    #define __TBB_TASK_ISOLATION (__TBB_CPF_BUILD||TBB_PREVIEW_TASK_ISOLATION)
+    #define __TBB_TASK_ISOLATION 1
 #endif /* __TBB_TASK_ISOLATION */
 
 #if TBB_PREVIEW_FLOW_GRAPH_TRACE
@@ -548,14 +551,8 @@ There are four cases that are supported:
 #endif /* __TBB_SURVIVE_THREAD_SWITCH */
 
 #ifndef __TBB_DEFAULT_PARTITIONER
-#if TBB_DEPRECATED
-/** Default partitioner for parallel loop templates in TBB 1.0-2.1 */
-#define __TBB_DEFAULT_PARTITIONER tbb::simple_partitioner
-#else
-/** Default partitioner for parallel loop templates since TBB 2.2 */
 #define __TBB_DEFAULT_PARTITIONER tbb::auto_partitioner
-#endif /* TBB_DEPRECATED */
-#endif /* !defined(__TBB_DEFAULT_PARTITIONER */
+#endif
 
 #ifndef __TBB_USE_PROPORTIONAL_SPLIT_IN_BLOCKED_RANGES
 #define __TBB_USE_PROPORTIONAL_SPLIT_IN_BLOCKED_RANGES 1
@@ -577,7 +574,7 @@ There are four cases that are supported:
     #endif
 #endif
 
-/** __TBB_WIN8UI_SUPPORT enables support of New Windows*8 Store Apps and limit a possibility to load
+/** __TBB_WIN8UI_SUPPORT enables support of Windows* Store Apps and limit a possibility to load
     shared libraries at run time only from application container **/
 #if defined(WINAPI_FAMILY) && WINAPI_FAMILY == WINAPI_FAMILY_APP
     #define __TBB_WIN8UI_SUPPORT 1
@@ -627,7 +624,7 @@ There are four cases that are supported:
 #endif
 
 #if (_WIN32||_WIN64) && __INTEL_COMPILER == 1110
-    /** That's a bug in Intel(R) C++ Compiler 11.1.044/IA-32 architecture/Windows* OS, that leads to a worker thread crash on the thread's startup. **/
+    /** That's a bug in Intel C++ Compiler 11.1.044/IA-32 architecture/Windows* OS, that leads to a worker thread crash on the thread's startup. **/
     #define __TBB_ICL_11_1_CODE_GEN_BROKEN 1
 #endif
 
@@ -658,7 +655,7 @@ There are four cases that are supported:
 #endif /* __FreeBSD__ */
 
 #if (__linux__ || __APPLE__) && __i386__ && defined(__INTEL_COMPILER)
-    /** The Intel(R) C++ Compiler for IA-32 architecture (Linux* OS|macOS) crashes or generates
+    /** The Intel C++ Compiler for IA-32 architecture (Linux* OS|macOS) crashes or generates
         incorrect code when __asm__ arguments have a cast to volatile. **/
     #define __TBB_ICC_ASM_VOLATILE_BROKEN 1
 #endif
@@ -686,11 +683,7 @@ There are four cases that are supported:
 #endif
 
 #if __INTEL_COMPILER==1300 && __TBB_GLIBCXX_VERSION>=40700 && defined(__GXX_EXPERIMENTAL_CXX0X__)
-/* Some C++11 features used inside libstdc++ are not supported by Intel compiler.
- * Checking version of gcc instead of libstdc++ because
- *  - they are directly connected,
- *  - for now it is not possible to check version of any standard library in this file
- */
+/* Some C++11 features used inside libstdc++ are not supported by Intel C++ Compiler. */
     #define __TBB_ICC_13_0_CPP11_STDLIB_SUPPORT_BROKEN 1
 #endif
 
@@ -703,7 +696,7 @@ There are four cases that are supported:
     #endif
 #endif
 
-/*In a PIC mode some versions of GCC 4.1.2 generate incorrect inlined code for 8 byte __sync_val_compare_and_swap intrinsic */
+/* In a PIC mode some versions of GCC 4.1.2 generate incorrect inlined code for 8 byte __sync_val_compare_and_swap intrinsic */
 #if __TBB_GCC_VERSION == 40102 && __PIC__ && !defined(__INTEL_COMPILER) && !defined(__clang__)
     #define __TBB_GCC_CAS8_BUILTIN_INLINING_BROKEN 1
 #endif
@@ -734,16 +727,18 @@ There are four cases that are supported:
 #define __TBB_IF_NO_COPY_CTOR_MOVE_SEMANTICS_BROKEN (_MSC_VER && (__INTEL_COMPILER >= 1300 && __INTEL_COMPILER <= 1310) && !__INTEL_CXX11_MODE__)
 
 #define __TBB_CPP11_DECLVAL_BROKEN (_MSC_VER == 1600 || (__GNUC__ && __TBB_GCC_VERSION < 40500) )
-
-// Intel C++ compiler has difficulties with copying std::pair with VC11 std::reference_wrapper being a const member
+// Intel C++ Compiler has difficulties with copying std::pair with VC11 std::reference_wrapper being a const member
 #define __TBB_COPY_FROM_NON_CONST_REF_BROKEN (_MSC_VER == 1700 && __INTEL_COMPILER && __INTEL_COMPILER < 1600)
 
 // The implicit upcasting of the tuple of a reference of a derived class to a base class fails on icc 13.X if the system's gcc environment is 4.8
 // Also in gcc 4.4 standard library the implementation of the tuple<&> conversion (tuple<A&> a = tuple<B&>, B is inherited from A) is broken.
-#if __GXX_EXPERIMENTAL_CXX0X__ && ((__INTEL_COMPILER >=1300 && __INTEL_COMPILER <=1310 && __TBB_GLIBCXX_VERSION>=40700) || (__TBB_GLIBCXX_VERSION < 40500))
+#if __GXX_EXPERIMENTAL_CXX0X__ && __GLIBCXX__ && ((__INTEL_COMPILER >=1300 && __INTEL_COMPILER <=1310 && __TBB_GLIBCXX_VERSION>=40700) || (__TBB_GLIBCXX_VERSION < 40500))
 #define __TBB_UPCAST_OF_TUPLE_OF_REF_BROKEN 1
 #endif
 
+// In some cases decltype of a function adds a reference to a return type.
+#define __TBB_CPP11_DECLTYPE_OF_FUNCTION_RETURN_TYPE_BROKEN (_MSC_VER == 1600 && !__INTEL_COMPILER)
+
 /** End of __TBB_XXX_BROKEN macro section **/
 
 #if defined(_MSC_VER) && _MSC_VER>=1500 && !defined(__INTEL_COMPILER)
@@ -764,7 +759,7 @@ There are four cases that are supported:
 
 #define __TBB_VARIADIC_PARALLEL_INVOKE          (TBB_PREVIEW_VARIADIC_PARALLEL_INVOKE && __TBB_CPP11_VARIADIC_TEMPLATES_PRESENT && __TBB_CPP11_RVALUE_REF_PRESENT)
 #define __TBB_FLOW_GRAPH_CPP11_FEATURES         (__TBB_CPP11_VARIADIC_TEMPLATES_PRESENT \
-                                                && __TBB_CPP11_RVALUE_REF_PRESENT && __TBB_CPP11_AUTO_PRESENT) \
+                                                && __TBB_CPP11_SMART_POINTERS_PRESENT && __TBB_CPP11_RVALUE_REF_PRESENT && __TBB_CPP11_AUTO_PRESENT) \
                                                 && __TBB_CPP11_VARIADIC_TUPLE_PRESENT && __TBB_CPP11_DEFAULT_FUNC_TEMPLATE_ARGS_PRESENT \
                                                 && !__TBB_UPCAST_OF_TUPLE_OF_REF_BROKEN
 #define __TBB_PREVIEW_STREAMING_NODE            (__TBB_CPP11_VARIADIC_FIXED_LENGTH_EXP_PRESENT && __TBB_FLOW_GRAPH_CPP11_FEATURES \
